@@ -40,7 +40,8 @@ const unsigned int batch_size = 10;
 const unsigned int warmup_size = 5;
 
 template<class T>
-std::vector<T> generate(size_t size, int entropy_reduction, int lower_level, int upper_level)
+std::vector<T>
+    generate(size_t size, int entropy_reduction, long long lower_level, long long upper_level)
 {
     if(entropy_reduction >= 5)
     {
@@ -52,21 +53,20 @@ std::vector<T> generate(size_t size, int entropy_reduction, int lower_level, int
     std::random_device rd;
     std::default_random_engine gen(rd());
     std::vector<T> data(size);
-    std::generate(
-        data.begin(), data.begin() + std::min(size, max_random_size),
-        [&]()
-        {
-            // Reduce enthropy by applying bitwise AND to random bits
-            // "An Improved Supercomputer Sorting Benchmark", 1992
-            // Kurt Thearling & Stephen Smith
-            auto v = gen();
-            for(int e = 0; e < entropy_reduction; e++)
-            {
-                v &= gen();
-            }
-            return T(lower_level + v % (upper_level - lower_level));
-        }
-    );
+    std::generate(data.begin(),
+                  data.begin() + std::min(size, max_random_size),
+                  [&]()
+                  {
+                      // Reduce entropy by applying bitwise AND to random bits
+                      // "An Improved Supercomputer Sorting Benchmark", 1992
+                      // Kurt Thearling & Stephen Smith
+                      auto v = gen();
+                      for(int e = 0; e < entropy_reduction; e++)
+                      {
+                          v &= gen();
+                      }
+                      return T(lower_level + v % (upper_level - lower_level));
+                  });
     for(size_t i = max_random_size; i < size; i += max_random_size)
     {
         std::copy_n(data.begin(), std::min(size - i, max_random_size), data.begin() + i);
@@ -99,8 +99,10 @@ void run_even_benchmark(benchmark::State& state,
 {
     using counter_type = unsigned int;
 
-    const int lower_level = 0;
-    const int upper_level = bins * scale;
+    const T lower_level = 0;
+    // casting for compilation with CUB backend because
+    // there is no casting from size_t (aka unsigned long) to __half
+    const T upper_level = static_cast<unsigned long long>(bins * scale);
 
     // Generate data
     std::vector<T> input = generate<T>(size, entropy_reduction, lower_level, upper_level);
@@ -392,9 +394,9 @@ void run_multi_range_benchmark(benchmark::State& state, size_t bins, hipStream_t
     const int num_levels_channel = bins + 1;
     int num_levels[ActiveChannels];
     std::vector<T> levels[ActiveChannels];
-    for (unsigned int channel = 0; channel < ActiveChannels; channel++) 
+    for (unsigned int channel = 0; channel < ActiveChannels; channel++)
     {
-        levels[channel].reserve(num_levels_channel);
+        levels[channel].resize(num_levels_channel);
         std::iota(levels[channel].begin(), levels[channel].end(), static_cast<T>(0));
         num_levels[channel] = num_levels_channel;
     }
@@ -489,23 +491,43 @@ void run_multi_range_benchmark(benchmark::State& state, size_t bins, hipStream_t
     }
 }
 
-#define CREATE_EVEN_BENCHMARK(T, BINS, SCALE) \
-benchmark::RegisterBenchmark( \
-    (std::string("histogram_even") + "<" #T ">" + \
-        "(" + std::to_string(get_entropy_percents(entropy_reduction)) + "% entropy, " + \
-        std::to_string(BINS) + " bins)" \
-    ).c_str(), \
-    [=](benchmark::State& state) { \
-        run_even_benchmark<T>(state, BINS, SCALE, entropy_reduction, stream, size); } \
-)
+template<class T>
+struct num_limits
+{
+    static constexpr T max()
+    {
+        return std::numeric_limits<T>::max();
+    };
+};
 
-#define BENCHMARK_TYPE(type) \
-    CREATE_EVEN_BENCHMARK(type, 10, 1234), \
-    CREATE_EVEN_BENCHMARK(type, 100, 1234), \
-    CREATE_EVEN_BENCHMARK(type, 1000, 1234), \
-    CREATE_EVEN_BENCHMARK(type, 16, 10), \
-    CREATE_EVEN_BENCHMARK(type, 256, 10), \
-    CREATE_EVEN_BENCHMARK(type, 65536, 1)
+template<>
+struct num_limits<__half>
+{
+    static constexpr double max()
+    {
+        return 65504.0;
+    };
+};
+
+#define CREATE_EVEN_BENCHMARK(VECTOR, T, BINS, SCALE)                                          \
+    if(num_limits<T>::max() > BINS * SCALE)                                                    \
+    {                                                                                          \
+        VECTOR.push_back(benchmark::RegisterBenchmark(                                         \
+            (std::string("histogram_even") + "<Datatype:" #T ">" + "(Entropy Percent:"         \
+             + std::to_string(get_entropy_percents(entropy_reduction)) + "%,Bin Count:"        \
+             + std::to_string(BINS) + " bins)")                                                \
+                .c_str(),                                                                      \
+            [=](benchmark::State& state)                                                       \
+            { run_even_benchmark<T>(state, BINS, SCALE, entropy_reduction, stream, size); })); \
+    }
+
+#define BENCHMARK_TYPE(VECTOR, T)                 \
+    CREATE_EVEN_BENCHMARK(VECTOR, T, 10, 1234);   \
+    CREATE_EVEN_BENCHMARK(VECTOR, T, 100, 1234);  \
+    CREATE_EVEN_BENCHMARK(VECTOR, T, 1000, 1234); \
+    CREATE_EVEN_BENCHMARK(VECTOR, T, 16, 10);     \
+    CREATE_EVEN_BENCHMARK(VECTOR, T, 256, 10);    \
+    CREATE_EVEN_BENCHMARK(VECTOR, T, 65536, 1)
 
 void add_even_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
                          hipStream_t stream,
@@ -513,21 +535,23 @@ void add_even_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmark
 {
     for(int entropy_reduction : entropy_reductions)
     {
-        std::vector<benchmark::internal::Benchmark*> bs =
-        {
-            BENCHMARK_TYPE(int),
-            BENCHMARK_TYPE(int8_t),
-            BENCHMARK_TYPE(uint8_t),
-            BENCHMARK_TYPE(unsigned short)
-        };
-        benchmarks.insert(benchmarks.end(), bs.begin(), bs.end());
+        BENCHMARK_TYPE(benchmarks, long long);
+        BENCHMARK_TYPE(benchmarks, int);
+        BENCHMARK_TYPE(benchmarks, unsigned short);
+        BENCHMARK_TYPE(benchmarks, uint8_t);
+        BENCHMARK_TYPE(benchmarks, double);
+        BENCHMARK_TYPE(benchmarks, float);
+        //this limitation can be removed once https://github.com/NVIDIA/cub/issues/484 is fixed
+#ifdef __HIP_PLATFORM_AMD__
+        BENCHMARK_TYPE(benchmarks, __half);
+#endif
     };
 }
 
 #define CREATE_MULTI_EVEN_BENCHMARK(CHANNELS, ACTIVE_CHANNELS, T, BINS, SCALE) \
 benchmark::RegisterBenchmark( \
-    (std::string("multi_histogram_even") + "<" #CHANNELS ", " #ACTIVE_CHANNELS ", " #T ">" + \
-        "(" + std::to_string(get_entropy_percents(entropy_reduction)) + "% entropy, " + \
+    (std::string("multi_histogram_even") + "<Channels:" #CHANNELS ",Active Channels:" #ACTIVE_CHANNELS ",Datatype:" #T ">" + \
+        "(Entropy Percent:" + std::to_string(get_entropy_percents(entropy_reduction)) + "%,Bin Count:" + \
         std::to_string(BINS) + " bins)" \
     ).c_str(), \
     [=](benchmark::State& state) { \
@@ -561,25 +585,23 @@ void add_multi_even_benchmarks(std::vector<benchmark::internal::Benchmark*>& ben
 
 #define CREATE_RANGE_BENCHMARK(T, BINS) \
 benchmark::RegisterBenchmark( \
-    (std::string("histogram_range") + "<" #T ">" + \
-        "(" + std::to_string(BINS) + " bins)" \
+    (std::string("histogram_range") + "<Datatype:" #T ">" + \
+        "(Bin Count:" + std::to_string(BINS) + " bins)" \
     ).c_str(), \
     [=](benchmark::State& state) { run_range_benchmark<T>(state, BINS, stream, size); } \
 )
 
+#define BENCHMARK_RANGE_TYPE(T)                                            \
+    CREATE_RANGE_BENCHMARK(T, 10), CREATE_RANGE_BENCHMARK(T, 100),         \
+        CREATE_RANGE_BENCHMARK(T, 1000), CREATE_RANGE_BENCHMARK(T, 10000), \
+        CREATE_RANGE_BENCHMARK(T, 100000), CREATE_RANGE_BENCHMARK(T, 1000000)
+
 void add_range_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
                           hipStream_t stream,
                           size_t size)
 {
-    std::vector<benchmark::internal::Benchmark*> bs =
-    {
-        CREATE_RANGE_BENCHMARK(float, 10),
-        CREATE_RANGE_BENCHMARK(float, 100),
-        CREATE_RANGE_BENCHMARK(float, 1000),
-        CREATE_RANGE_BENCHMARK(float, 10000),
-        CREATE_RANGE_BENCHMARK(float, 100000),
-        CREATE_RANGE_BENCHMARK(float, 1000000),
-    };
+    std::vector<benchmark::internal::Benchmark*> bs
+        = {BENCHMARK_RANGE_TYPE(float), BENCHMARK_RANGE_TYPE(double)};
     benchmarks.insert(benchmarks.end(), bs.begin(), bs.end());
 }
 
@@ -623,6 +645,8 @@ int main(int argc, char *argv[])
     const size_t size = parser.get<size_t>("size");
     const int trials = parser.get<int>("trials");
 
+    std::cout << "benchmark_device_histogram" << std::endl;
+
     // HIP
     hipStream_t stream = 0; // default
     hipDeviceProp_t devProp;
