@@ -345,8 +345,8 @@ void run_sort_pairs_benchmark(benchmark::State &state,
 
 #define CREATE_SORT_KEYS_BENCHMARK(Key, SEGMENTS)        \
     benchmark::RegisterBenchmark(                        \
-        (std::string("sort_keys") + "<" #Key ">" +       \
-         "(~" + std::to_string(SEGMENTS) + " segments)") \
+        (std::string("sort_keys") + "<Key Type:" #Key ">" +       \
+         "(Number of segments:~" + std::to_string(SEGMENTS) + " segments)") \
             .c_str(),                                    \
         [=](benchmark::State &state) { run_sort_keys_benchmark<Key>(state, SEGMENTS, stream, size); }), \
     benchmark::RegisterBenchmark(                        \
@@ -388,8 +388,8 @@ void add_sort_keys_benchmarks(std::vector<benchmark::internal::Benchmark *> &ben
 
 #define CREATE_SORT_PAIRS_BENCHMARK(Key, Value, SEGMENTS)       \
     benchmark::RegisterBenchmark(                               \
-        (std::string("sort_pairs") + "<" #Key ", " #Value ">" + \
-         "(~" + std::to_string(SEGMENTS) + " segments)")        \
+        (std::string("sort_pairs") + "<Key Type:" #Key ",Value Type:" #Value ">" + \
+         "(Number of segments:~" + std::to_string(SEGMENTS) + " segments)")        \
             .c_str(),                                           \
         [=](benchmark::State &state) { run_sort_pairs_benchmark<Key, Value>(state, SEGMENTS, stream, size); }), \
     benchmark::RegisterBenchmark(                               \
@@ -444,6 +444,8 @@ int main(int argc, char *argv[])
     const size_t size = parser.get<size_t>("size");
     const int trials = parser.get<int>("trials");
 
+    std::cout << "benchmark_device_segmented_sort" << std::endl;
+
     // HIP
     hipStream_t stream = 0; // default
     hipDeviceProp_t devProp;
