@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2020 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -71,7 +71,7 @@ struct custom_flag_op2
     }
 };
 
-// Host (CPU) implementaions of the wrapping function that allows to pass 3 args
+// Host (CPU) implementations of the wrapping function that allows to pass 3 args
 template<class T, class FlagType, class FlagOp>
 typename std::enable_if<hipcub::detail::WithBIndexArg<T, FlagOp>::value, FlagType>::type
 apply(FlagOp flag_op, const T& a, const T& b, unsigned int b_index)
@@ -86,35 +86,48 @@ apply(FlagOp flag_op, const T& a, const T& b, unsigned int)
     return flag_op(b, a);
 }
 
-
-
 typedef ::testing::Types<
     // Power of 2 BlockSize
     params<unsigned int, int, 64U, 1, hipcub::Equality>,
     params<int, bool, 128U, 1, hipcub::Inequality>,
     params<float, int, 256U, 1, test_utils::less>,
+#ifndef __HIP_PLATFORM_NVIDIA__
+    // Nvidia does not handle half and bfloat16 properly
+    params<test_utils::half, int, 256U, 1, test_utils::less>,
+    params<test_utils::bfloat16, int, 256U, 1, test_utils::less>,
+#endif
     params<char, char, 1024U, 1, test_utils::less_equal>,
-    params<int, bool, 256U, 1, custom_flag_op1<int> >,
+    params<int, bool, 256U, 1, custom_flag_op1<int>>,
 
     // Non-power of 2 BlockSize
     params<double, unsigned int, 65U, 1, test_utils::greater>,
-    params<float, int, 37U, 1, custom_flag_op1<float> >,
+    params<float, int, 37U, 1, custom_flag_op1<float>>,
+
+#ifndef __HIP_PLATFORM_NVIDIA__
+    params<test_utils::half, int, 37U, 1, test_utils::greater>,
+    params<test_utils::bfloat16, int, 37U, 1, test_utils::greater>,
+#endif
+
     params<long long, char, 510U, 1, test_utils::greater_equal>,
     params<unsigned int, long long, 162U, 1, hipcub::Inequality>,
     params<unsigned char, bool, 255U, 1, hipcub::Equality>,
 
     // Power of 2 BlockSize and ItemsPerThread > 1
-    params<int, char, 64U, 2, custom_flag_op2<int> >,
+    params<int, char, 64U, 2, custom_flag_op2<int>>,
     params<int, short, 128U, 4, test_utils::less>,
-    params<unsigned short, unsigned char, 256U, 7, custom_flag_op2<unsigned short> >,
+    params<unsigned short, unsigned char, 256U, 7, custom_flag_op2<unsigned short>>,
     params<short, short, 512U, 8, hipcub::Equality>,
 
     // Non-power of 2 BlockSize and ItemsPerThread > 1
-    params<double, int, 33U, 5, custom_flag_op2<double> >,
+    params<double, int, 33U, 5, custom_flag_op2<double>>,
     params<double, unsigned int, 464U, 2, hipcub::Equality>,
+#ifndef __HIP_PLATFORM_NVIDIA__
+    params<test_utils::half, int, 37U, 5, test_utils::greater>,
+    params<test_utils::bfloat16, int, 37U, 3, test_utils::greater>,
+#endif
     params<unsigned short, int, 100U, 3, test_utils::greater>,
-    params<short, bool, 234U, 9, custom_flag_op1<short> >
-> Params;
+    params<short, bool, 234U, 9, custom_flag_op1<short>>>
+    Params;
 
 TYPED_TEST_SUITE(HipcubBlockAdjacentDifference, Params);
 
@@ -284,27 +297,30 @@ struct custom_op2
     }
 };
 
-typedef ::testing::Types<
-    params_subtract<unsigned int, int, hipcub::Sum, 64U, 1>,
-    params_subtract<int, bool, custom_op1, 128U, 1>,
-    params_subtract<float, int, custom_op2, 256U, 1>,
-    params_subtract<int, bool, custom_op1, 256U, 1>,
+typedef ::testing::Types<params_subtract<unsigned int, int, hipcub::Sum, 64U, 1>,
+                         params_subtract<int, bool, custom_op1, 128U, 1>,
+                         params_subtract<float, int, custom_op2, 256U, 1>,
+#ifndef __HIP_PLATFORM_NVIDIA__
+                         params_subtract<test_utils::half, int, custom_op1, 256U, 1>,
+                         params_subtract<test_utils::bfloat16, int, custom_op2, 256U, 1>,
+#endif
+                         params_subtract<int, bool, custom_op1, 256U, 1>,
 
-    params_subtract<float, int, hipcub::Sum, 37U, 1>,
-    params_subtract<long long, char, custom_op1, 510U, 1>,
-    params_subtract<unsigned int, long long, custom_op2, 162U, 1>,  
-    params_subtract<unsigned char, bool, hipcub::Sum, 255U, 1>,
+                         params_subtract<float, int, hipcub::Sum, 37U, 1>,
+                         params_subtract<long long, char, custom_op1, 510U, 1>,
+                         params_subtract<unsigned int, long long, custom_op2, 162U, 1>,
+                         params_subtract<unsigned char, bool, hipcub::Sum, 255U, 1>,
 
-    params_subtract<int, char, custom_op1, 64U, 2>,
-    params_subtract<int, short, custom_op2, 128U, 4>,
-    params_subtract<unsigned short, unsigned char, hipcub::Sum, 256U, 7>,
-    params_subtract<short, short, custom_op1, 512U, 8>,
+                         params_subtract<int, char, custom_op1, 64U, 2>,
+                         params_subtract<int, short, custom_op2, 128U, 4>,
+                         params_subtract<unsigned short, unsigned char, hipcub::Sum, 256U, 7>,
+                         params_subtract<short, short, custom_op1, 512U, 8>,
 
-    params_subtract<double, int, custom_op2, 33U, 5>,
-    params_subtract<double, unsigned int, hipcub::Sum, 464U, 2>,
-    params_subtract<unsigned short, int, custom_op1, 100U, 3>,
-    params_subtract<short, bool, custom_op2, 234U, 9>
-> ParamsSubtract;
+                         params_subtract<double, int, custom_op2, 33U, 5>,
+                         params_subtract<double, unsigned int, hipcub::Sum, 464U, 2>,
+                         params_subtract<unsigned short, int, custom_op1, 100U, 3>,
+                         params_subtract<short, bool, custom_op2, 234U, 9>>
+    ParamsSubtract;
 
 TYPED_TEST_SUITE(HipcubBlockAdjacentDifferenceSubtract, ParamsSubtract);
 
@@ -368,8 +384,23 @@ void subtract_left_partial_tile_kernel(const T* input, int* tile_sizes, StorageT
     Output thread_output[ItemsPerThread];
 
     int tile_size = tile_sizes[blockIdx.x];
-    
-    adjacent_difference.SubtractLeftPartialTile(thread_items, thread_output, BinaryFunction{}, tile_size);
+
+    if(blockIdx.x % 2 == 1)
+    {
+        const T tile_predecessor_item = input[block_offset - 1];
+        adjacent_difference.SubtractLeftPartialTile(thread_items,
+                                                    thread_output,
+                                                    BinaryFunction{},
+                                                    tile_size,
+                                                    tile_predecessor_item);
+    }
+    else
+    {
+        adjacent_difference.SubtractLeftPartialTile(thread_items,
+                                                    thread_output,
+                                                    BinaryFunction{},
+                                                    tile_size);
+    }
 
     hipcub::StoreDirectBlocked(lid, output + block_offset, thread_output);
 }
@@ -442,20 +473,23 @@ void subtract_right_partial_tile_kernel(const T* input, int* tile_sizes, Storage
 
 TYPED_TEST(HipcubBlockAdjacentDifference, FlagHeads)
 {
-  using type = typename TestFixture::params::type;
-  // std::vector<bool> is a special case that will cause an error in hipMemcpy
-  using stored_flag_type = typename std::conditional<
-                             std::is_same<bool, typename TestFixture::params::flag_type>::value,
-                             int,
-                             typename TestFixture::params::flag_type
-                         >::type;
-  using flag_type = typename TestFixture::params::flag_type;
-  using flag_op_type = typename TestFixture::params::flag_op_type;
-  constexpr size_t block_size = TestFixture::params::block_size;
-  constexpr size_t items_per_thread = TestFixture::params::items_per_thread;
-  constexpr size_t items_per_block = block_size * items_per_thread;
-  const size_t size = items_per_block * 2048;
-  constexpr size_t grid_size = size / items_per_block;
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using type = typename TestFixture::params::type;
+    // std::vector<bool> is a special case that will cause an error in hipMemcpy
+    using stored_flag_type = typename std::conditional<
+        std::is_same<bool, typename TestFixture::params::flag_type>::value,
+        int,
+        typename TestFixture::params::flag_type>::type;
+    using flag_type                   = typename TestFixture::params::flag_type;
+    using flag_op_type                = typename TestFixture::params::flag_op_type;
+    constexpr size_t block_size       = TestFixture::params::block_size;
+    constexpr size_t items_per_thread = TestFixture::params::items_per_thread;
+    constexpr size_t items_per_block  = block_size * items_per_thread;
+    const size_t     size             = items_per_block * 2048;
+    constexpr size_t grid_size        = size / items_per_block;
 
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
@@ -544,20 +578,23 @@ TYPED_TEST(HipcubBlockAdjacentDifference, FlagHeads)
 
 TYPED_TEST(HipcubBlockAdjacentDifference, FlagTails)
 {
-  using type = typename TestFixture::params::type;
-  // std::vector<bool> is a special case that will cause an error in hipMemcpy
-  using stored_flag_type = typename std::conditional<
-                             std::is_same<bool, typename TestFixture::params::flag_type>::value,
-                             int,
-                             typename TestFixture::params::flag_type
-                         >::type;
-  using flag_type = typename TestFixture::params::flag_type;
-  using flag_op_type = typename TestFixture::params::flag_op_type;
-  constexpr size_t block_size = TestFixture::params::block_size;
-  constexpr size_t items_per_thread = TestFixture::params::items_per_thread;
-  constexpr size_t items_per_block = block_size * items_per_thread;
-  const size_t size = items_per_block * 2048;
-  constexpr size_t grid_size = size / items_per_block;
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using type = typename TestFixture::params::type;
+    // std::vector<bool> is a special case that will cause an error in hipMemcpy
+    using stored_flag_type = typename std::conditional<
+        std::is_same<bool, typename TestFixture::params::flag_type>::value,
+        int,
+        typename TestFixture::params::flag_type>::type;
+    using flag_type                   = typename TestFixture::params::flag_type;
+    using flag_op_type                = typename TestFixture::params::flag_op_type;
+    constexpr size_t block_size       = TestFixture::params::block_size;
+    constexpr size_t items_per_thread = TestFixture::params::items_per_thread;
+    constexpr size_t items_per_block  = block_size * items_per_thread;
+    const size_t     size             = items_per_block * 2048;
+    constexpr size_t grid_size        = size / items_per_block;
 
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
@@ -646,20 +683,23 @@ TYPED_TEST(HipcubBlockAdjacentDifference, FlagTails)
 
 TYPED_TEST(HipcubBlockAdjacentDifference, FlagHeadsAndTails)
 {
-  using type = typename TestFixture::params::type;
-  // std::vector<bool> is a special case that will cause an error in hipMemcpy
-  using stored_flag_type = typename std::conditional<
-                             std::is_same<bool, typename TestFixture::params::flag_type>::value,
-                             int,
-                             typename TestFixture::params::flag_type
-                         >::type;
-  using flag_type = typename TestFixture::params::flag_type;
-  using flag_op_type = typename TestFixture::params::flag_op_type;
-  constexpr size_t block_size = TestFixture::params::block_size;
-  constexpr size_t items_per_thread = TestFixture::params::items_per_thread;
-  constexpr size_t items_per_block = block_size * items_per_thread;
-  const size_t size = items_per_block * 2048;
-  constexpr size_t grid_size = size / items_per_block;
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using type = typename TestFixture::params::type;
+    // std::vector<bool> is a special case that will cause an error in hipMemcpy
+    using stored_flag_type = typename std::conditional<
+        std::is_same<bool, typename TestFixture::params::flag_type>::value,
+        int,
+        typename TestFixture::params::flag_type>::type;
+    using flag_type                   = typename TestFixture::params::flag_type;
+    using flag_op_type                = typename TestFixture::params::flag_op_type;
+    constexpr size_t block_size       = TestFixture::params::block_size;
+    constexpr size_t items_per_thread = TestFixture::params::items_per_thread;
+    constexpr size_t items_per_block  = block_size * items_per_thread;
+    const size_t     size             = items_per_block * 2048;
+    constexpr size_t grid_size        = size / items_per_block;
 
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
@@ -772,6 +812,10 @@ TYPED_TEST(HipcubBlockAdjacentDifference, FlagHeadsAndTails)
 
 TYPED_TEST(HipcubBlockAdjacentDifferenceSubtract, SubtractLeft)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
     using type = typename TestFixture::params_subtract::type;
     using binary_function = typename TestFixture::params_subtract::binary_function;
 
@@ -867,6 +911,10 @@ TYPED_TEST(HipcubBlockAdjacentDifferenceSubtract, SubtractLeft)
 
 TYPED_TEST(HipcubBlockAdjacentDifferenceSubtract, SubtractLeftPartialTile)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
     using type = typename TestFixture::params_subtract::type;
     using binary_function = typename TestFixture::params_subtract::binary_function;
 
@@ -912,7 +960,8 @@ TYPED_TEST(HipcubBlockAdjacentDifferenceSubtract, SubtractLeftPartialTile)
                 {
                     if(item == 0) 
                     {
-                        expected[i] = static_cast<output_type>(input[i]);
+                        expected[i] = static_cast<output_type>(
+                            block_index % 2 == 1 ? op(input[i], input[i - 1]) : input[i]);
                     } 
                     else 
                     {
@@ -981,6 +1030,10 @@ TYPED_TEST(HipcubBlockAdjacentDifferenceSubtract, SubtractLeftPartialTile)
 
 TYPED_TEST(HipcubBlockAdjacentDifferenceSubtract, SubtractRight)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
     using type = typename TestFixture::params_subtract::type;
     using binary_function = typename TestFixture::params_subtract::binary_function;
 
@@ -1076,6 +1129,10 @@ TYPED_TEST(HipcubBlockAdjacentDifferenceSubtract, SubtractRight)
 
 TYPED_TEST(HipcubBlockAdjacentDifferenceSubtract, SubtractRightPartialTile)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
     using type = typename TestFixture::params_subtract::type;
     using binary_function = typename TestFixture::params_subtract::binary_function;
 
@@ -1186,4 +1243,4 @@ TYPED_TEST(HipcubBlockAdjacentDifferenceSubtract, SubtractRightPartialTile)
         HIP_CHECK(hipFree(d_tile_sizes));
         HIP_CHECK(hipFree(d_output));
     }
-}
\ No newline at end of file
+}
