@@ -188,6 +188,10 @@ TYPED_TEST_SUITE(HipcubIteratorTests, HipcubIteratorTestsParams);
 
 TYPED_TEST(HipcubIteratorTests, TestCacheModifiedInput)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
     using T = typename TestFixture::input_type;
     using IteratorType = hipcub::CacheModifiedInputIterator<hipcub::LOAD_CG, T>;
 
@@ -223,6 +227,10 @@ TYPED_TEST(HipcubIteratorTests, TestCacheModifiedInput)
 
 TYPED_TEST(HipcubIteratorTests, TestConstant)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
     using T = typename TestFixture::input_type;
     using IteratorType = hipcub::ConstantInputIterator<T>;
 
@@ -247,6 +255,10 @@ TYPED_TEST(HipcubIteratorTests, TestConstant)
 
 TYPED_TEST(HipcubIteratorTests, TestCounting)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
     using T = typename TestFixture::input_type;
     using IteratorType = hipcub::CountingInputIterator<T>;
 
@@ -275,6 +287,10 @@ TYPED_TEST(HipcubIteratorTests, TestCounting)
 
 TYPED_TEST(HipcubIteratorTests, TestTransform)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
     using T = typename TestFixture::input_type;
     using CastT = typename TestFixture::input_type;
     using IteratorType = hipcub::TransformInputIterator<T, TransformOp<T>, CastT*>;
@@ -322,6 +338,20 @@ TYPED_TEST(HipcubIteratorTests, TestTransform)
 
 TYPED_TEST(HipcubIteratorTests, TestTexObj)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+
+    hipDeviceProp_t props;
+    HIP_CHECK(hipGetDeviceProperties(&props, device_id));
+    std::string deviceName = std::string(props.gcnArchName);
+    if (deviceName.rfind("gfx94", 0) == 0) {
+        // This is a gfx94x device, so skip this test
+        SCOPED_TRACE(testing::Message() << "Skipping texture cache text for " << deviceName);
+        return;
+    }
+
+    HIP_CHECK(hipSetDevice(device_id));
+
     using T = typename TestFixture::input_type;
     using CastT = typename TestFixture::input_type;
     using IteratorType = hipcub::TexObjInputIterator<T>;
@@ -376,6 +406,20 @@ TYPED_TEST(HipcubIteratorTests, TestTexObj)
 
 TYPED_TEST(HipcubIteratorTests, TestTexRef)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+
+    hipDeviceProp_t props;
+    HIP_CHECK(hipGetDeviceProperties(&props, device_id));
+    std::string deviceName = std::string(props.gcnArchName);
+    if (deviceName.rfind("gfx94", 0) == 0) {
+        // This is a gfx94x device, so skip this test
+        SCOPED_TRACE(testing::Message() << "Skipping texture cache text for " << deviceName);
+        return;
+    }
+
+    HIP_CHECK(hipSetDevice(device_id));
+
     using T = typename TestFixture::input_type;
     using CastT = typename TestFixture::input_type;
     using IteratorType = hipcub::TexRefInputIterator<T, __LINE__>;
@@ -421,7 +465,7 @@ TYPED_TEST(HipcubIteratorTests, TestTexRef)
         IteratorType d_ref_itr;
         d_ref_itr.BindTexture((CastT*) d_data, sizeof(T) * TEST_VALUES);
 
-        // Create and bind dummy iterator of same type to check with interferance
+        // Create and bind dummy iterator of same type to check with interference
         IteratorType d_ref_itr2;
         d_ref_itr2.BindTexture((CastT*) d_dummy, sizeof(T) * DUMMY_TEST_VALUES);
 
@@ -434,6 +478,20 @@ TYPED_TEST(HipcubIteratorTests, TestTexRef)
 
 TYPED_TEST(HipcubIteratorTests, TestTexTransform)
 {
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+
+    hipDeviceProp_t props;
+    HIP_CHECK(hipGetDeviceProperties(&props, device_id));
+    std::string deviceName = std::string(props.gcnArchName);
+    if (deviceName.rfind("gfx94", 0) == 0) {
+        // This is a gfx94x device, so skip this test
+        SCOPED_TRACE(testing::Message() << "Skipping texture cache text for " << deviceName);
+        return;
+    }
+
+    HIP_CHECK(hipSetDevice(device_id));
+
     using T = typename TestFixture::input_type;
     using CastT = typename TestFixture::input_type;
     using TextureIteratorType = hipcub::TexRefInputIterator<T, __LINE__>;
